/*
 * Octopus Load Balancer - Configuration file parser
 *
 * Copyright 2008 Alistair Reay <alreay1@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 */


int parse_config_file(const char *conf_file) {
	int status;
	FILE *fp;
	char *directive;
	char *value;
	char buffer[1000];
	int inServerSection=0;
	int serverStatus=1;
	int serverPort=80;
	int serverMaxc=group->default_maxc;
	float serverMaxload=group->default_maxl;
	struct in_addr serverIP;
	char serverName[SERVERNAME_MAX_LENGTH];
	int serverClone=0;
	int lineCounter=0;
	int v1;
	char *c1;
	errno=0;
	if((fp=fopen(conf_file,"r"))!=NULL) {
		while(fgets(buffer, 1000, fp)!=NULL) {
			lineCounter++;
			directive = strtok(buffer,"=");
			value = strtok(NULL," =");

			if (inServerSection) {
				if (!strncmp(directive, "port",4)) {
					errno=0;
					serverPort=strtol(value, (char **)NULL, 10);
					if (errno !=0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: port value invalid: %s", lineCounter, strerror(errno));
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(!((serverPort > 0) && (serverPort < 65536))) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting port to: %d",serverPort);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				if (!strncmp(directive, "maxc",4)) {
					errno=0;
					serverMaxc=strtol(value, (char **)NULL, 10);
					if (errno !=0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: maxc value invalid: %s", lineCounter, strerror(errno));
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(!(serverMaxc >= 0)) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: maxc value invalid", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting maxc to: %d",serverMaxc);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				if (!strncmp(directive, "maxload",7)) {
					serverMaxload=atof(value);
					if(!(serverMaxload > 0.0)) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: maxload value invalid", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting maxload to: %f",serverMaxload);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				if (!strncmp(directive, "status",5)) {
					errno=0;
					serverStatus=strtol(value, (char **)NULL, 10);
					if (errno !=0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: status value invalid: %s", lineCounter, strerror(errno));
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(!((serverStatus >= 0) && (serverStatus <= 1))) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: status value invalid", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting status to: %d",serverStatus);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}

				if (!strncmp(directive, "ip",2)) {
					status=inet_aton(value, &serverIP);
					if(status == 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: IP invalid", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting IP to: %s", inet_ntoa(serverIP));
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				if (!strncmp(directive, "clone",5)) {
					errno=0;
					serverClone=strtol(value, (char **)NULL, 10);
					if (errno !=0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: clone value invalid: %s", lineCounter, strerror(errno));
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(!((serverClone >= 0) && (serverClone <= 1))) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parse_config_file: line %d: clone value invalid: %s", lineCounter, strerror(errno));
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting server clone value to: %d",serverClone);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}

			}

			if (!strncmp(directive, "[/",2)) {
				if(debug_level > 0) {
					directive[strlen(directive) -1] = '\0';
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: found end-of-section: %s",directive);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
				inServerSection=0;
				status = create_group_server(serverClone, serverName, serverStatus, serverPort, &serverIP, serverMaxc, serverMaxload);
				if(status < 0) {
					if(status==-1) {
						fprintf(stderr, "ERROR: memory allocation error\n");
					}
					if(status==-2) {
						fprintf(stderr, "ERROR: invalid tcp port number\n");
					}
					if(status==-3) {
						fprintf(stderr, "ERROR: invalid IP address\n");
					}
					fprintf(stderr, "FATAL ERROR: unable to create server %s\n", serverName);
					fprintf(stderr, "  -status  = %d\n", serverStatus);
					fprintf(stderr, "  -port    = %d\n", serverPort);
					fprintf(stderr, "  -IP      = %s", inet_ntoa(serverIP));
					fprintf(stderr, "  -Maxc    = %d\n", serverMaxc);
					fprintf(stderr, "  -Maxload = %f\n", serverMaxload);
					exit(1);
				}
				/* reset the values for the next server */
				serverStatus=1;
				serverPort=80;
				serverClone=0;
				serverMaxc=group->default_maxc;
				serverMaxload=group->default_maxl;
				continue;
			}
			if (!strncmp(directive, "[",1)) {
				directive[strlen(directive) -1]='\0';
				inServerSection=1;
				memset(&serverName, '\0', SERVERNAME_MAX_LENGTH);
				memcpy(serverName, directive+1, strlen(directive)-2);
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: found start-of-section: %s",directive);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
				continue;
			}
			if (!strncmp(directive, "snmp_community_pw", 17)) {
#ifdef USE_SNMP
				group->snmp_community_pw = malloc(sizeof(u_char) * strlen(value));
				if(group->snmp_community_pw != NULL) {
					memcpy(group->snmp_community_pw, value, strlen(value));
					group->snmp_community_pw[strlen(value)-1]='\0';
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting snmp community password to \"%s\"", group->snmp_community_pw);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				else {
					write_log(OCTOPUS_LOG_EXIT, "Unable to allocate memory for snmp_community_pw buffer!");
				}
#else
				snprintf(log_string, OCTOPUS_LOG_LEN, "NOTICE: parse_config_file: snmp_community_pw has no effect unless Octopus is compiled with SNMP development libraries");
				write_log(OCTOPUS_LOG_STD, log_string);
#endif
			}

			if (!strncmp(directive, "snmp_enable", 11)) {
#ifdef USE_SNMP
				if(!strncmp(value, "on", 2)) {
					group->snmp_status=SNMP_ENABLED;
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting snmp status to enabled");
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
				else {
					group->snmp_status=SNMP_DISABLED;
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting snmp status to disabled");
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}
#else
				snprintf(log_string, OCTOPUS_LOG_LEN, "NOTICE: parse_config_file: enabling snmp monitoring has no effect unless Octopus is compiled with SNMP development libraries");
				write_log(OCTOPUS_LOG_STD, log_string);
#endif
			}


			if (!strncmp(directive, "shm_run_dir", 11)) {
				group->shm_run_dir= malloc(strlen(value));
				if(group->shm_run_dir != NULL) {
					memcpy(group->shm_run_dir, value, strlen(value));
					group->shm_run_dir[strlen(value)-1]='\0';
					if(debug_level > 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting shm run dir to %s", group->shm_run_dir);
						write_log(OCTOPUS_LOG_STD, log_string);
					}
				}

				else {
					write_log(OCTOPUS_LOG_EXIT, "Unable to allocate memory for shm_run_dir buffer!");
				}
			}
			if (!strncmp(directive, "session_weight", 14)) {
				group->session_weight=atof(value);
				if(!(group->session_weight >= 0.0)) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: session_weight value invalid, must be greater than or equal to zero", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}

				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting session_weight to %f", group->session_weight);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "algorithm",9)) {
				if (!strncmp(value, "RR", 2)) {
					group->algorithm= ALGORITHM_RR;
					group->choose_server=choose_rr_server;
				}
				else if (!strncmp(value, "LC", 2)) {
					group->algorithm= ALGORITHM_LC;
					group->choose_server=choose_lc_server;
				}
				else if (!strncmp(value, "LL", 2)) {
					#ifndef USE_SNMP
					fprintf(stderr, "ERROR: Cannot use algoritithm Least Load unless compiled with SNMP developlment libraries");
					exit(1);
					#endif
					group->algorithm= ALGORITHM_LL;
					group->choose_server=choose_ll_server;
				}
				else if (!strncmp(value, "HASH", 4)) {
					group->algorithm= ALGORITHM_HASH;
					group->choose_server=choose_hash_server;
				}
				else {
					fprintf(stderr, "ERROR: parsing config file at line %d: unsupported allocation algorithm!!", lineCounter);
					exit(1);
				}
			}
			if (!strncmp(directive, "clone_mode",10)) {
				if(!strncmp(value, "OFF", 3)) {
					group->clone_mode=MODE_OFF;
				}
				else if(!strncmp(value, "FAST",4)) {
					group->clone_mode=MODE_FAST;
				}
				else if(!strncmp(value, "STRICT", 5)) {
					group->clone_mode=MODE_STRICT;
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: clone_mode value invalid", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting clone_mode to: %d",group->clone_mode);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "overload_mode", 13)) {
				if(!strncmp(value, "STRICT", 5)) {
					group->overload_mode=MODE_STRICT;
				}
				else if(!strncmp(value, "RELAXED", 5)) {
					group->overload_mode=MODE_RELAXED;
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: overload_mode value invalid", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting overload_mode to: %d",group->overload_mode);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "hash_rebalance_threshold", 24)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if((v1 < 0) || (v1 >= 100)) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_threshold value invalid, must be greater than or equal to 0 and less than 100", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->hash_rebalance_threshold= v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_threshold value invalid, must be greater than or equal to 0 and less than 100", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting hash_rebalance_threshold to: %d",group->hash_rebalance_threshold);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "hash_rebalance_size", 19)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if((v1 < 0) || (v1 >= 100)) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_size value invalid, must be greater than or equal to 0 and less than 100", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
						continue;
					}
					else {
						group->hash_rebalance_size= v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_size value invalid, must be greater than or equal to 0 and less than 100", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting hash_rebalance_size to: %d",group->hash_rebalance_size);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "hash_rebalance_interval", 21)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 < 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_interval value invalid, must be greater than or equal to zero", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
						continue;
					}
					else {
						group->hash_rebalance_interval= v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: hash_rebalance_interval value invalid, must be greater than or equal to zero", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}

				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting hash_rebalance_interval to: %d",group->hash_rebalance_interval);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "shm_perms", 9)) {
				v1=strtol(value, &c1, 8);
				if(value != c1) {
					if(v1 <= 0 || v1 >= 777) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: shm_perms value invalid, must be between 0 and 777", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->shm_perms=v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: shm_perms value invalid, must be between 0 and 777", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting shm_perms to: %li",strtol(value, &c1, 10));
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "monitor_interval",16)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 < 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: monitor_interval value invalid, must be greater than or equal to 0", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->monitor_interval=v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: monitor_interval value invalid, must be greater than or equal to 0", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting monitor_interval to: %d",group->monitor_interval);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "connect_timeout",15)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 <= 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: connect_timeout value invalid, must be greater than zero", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->connect_timeout=v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: connect_timeout value invalid, must be greater than zero", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting connect_timeout to: %d",group->connect_timeout);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "session_limit",13)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 < 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: session_limit value invalid, must be greater than or equal to 0 ", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->session_limit=v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: session_limit value invalid, must be greater than or equal to 0 ", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}

				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting session_limit to: %d",group->connect_timeout);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "fd_limit",8)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 < 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: fd_limit value invalid, must be greater than or equal to 0", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->fd_limit=v1;
					}
				}
				else {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: fd_limit value invalid, must be greater than or equal to 0", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting fd_limit to: %d",group->fd_limit);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}

			if (!strncmp(directive, "use_syslog",10)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if((v1 < 0) || (v1 >1)) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: use_syslog value invalid, must be either 0 or 1", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->use_syslog=v1;
					}
				}
				else {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: use_syslog value invalid, must be either 0 or 1", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting use_syslog to: %d",group->use_syslog);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "binding_port",12)) {
				v1=strtol(value, &c1, 10);
	            if(value != c1) {
					if(v1 <= 0) {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: binding_port value invalid, must be greater than 0", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
					}
					else {
						group->binding_port=v1;
					}
				}
				else {
						snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: binding_port value invalid, must be greater than 0", lineCounter);
						write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: parse_config_file: setting binding_port to: %d",group->binding_port);
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "binding_ip",10)) {
				if(inet_aton(value, &group->binding_ip) ==0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"ERROR: parsing config file at line %d: binding IP value invalid", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting binding_ip to: %s",inet_ntoa(group->binding_ip));
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "member_outbound_ip",18)) {
				if(inet_aton(value, &group->member_outbound_ip) ==0) {
					snprintf(log_string, OCTOPUS_LOG_LEN,"ERROR: parsing config file at line %d: member_outbound_ip value invalid", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				group->use_member_outbound_ip=1;
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting member_outbound_ip to: %s",inet_ntoa(group->member_outbound_ip));
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
			if (!strncmp(directive, "clone_outbound_ip",17)) {
				if(inet_aton(value, &group->clone_outbound_ip) ==0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: parsing config file at line %d: clone_outbound_ip value invalid", lineCounter);
					write_log(OCTOPUS_LOG_EXIT, log_string);
				}
				group->use_clone_outbound_ip=1;
				if(debug_level > 0) {
					snprintf(log_string, OCTOPUS_LOG_LEN, "DEBUG: parse_config_file: setting clone_outbound_ip to: %s",inet_ntoa(group->clone_outbound_ip));
					write_log(OCTOPUS_LOG_STD, log_string);
				}
			}
		}
		fclose(fp);
		return 0;
	}
	else {
		snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR! unable to open conf file! - %s", strerror(errno));
		write_log(OCTOPUS_LOG_EXIT, log_string);
	}
	return -1;

}

