/*
 * Octopus Load Balancer - Signal handler and shutdown function.
 *
 * Copyright 2008 Alistair Reay <alreay1@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 */

static void signal_handler(int signum) {
	switch(signum) {
		case SIGTERM:
			clean_exit();
		case SIGINT:
			clean_exit();
	}
}
int clean_exit() {
	int status;
	pid_t p;
	if(getpid() == group->monitor_pid) {
		if(group->shmid != -1) {
			if(debug_level > 0) {
				write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: monitor: received signal SIGTERM. Shutting down...");
			}
			status=shmdt((const void *)group);
			if(status !=0) {
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: monitor: unable to detach SHM segment: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
			}
		}
		else {
			write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: monitor: termination signal received, no SHM segment detected so not detaching");
		}
	}
	if(getpid() == group->master_pid) {
		write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master: received signal SIGTERM. Shutting down...");
		group->alive=0;

		/* wait for monitoring process to quit first */
		//p = waitpid(group->monitor_pid, &status, 0);
		p = waitpid(-1, &status, 0);
		if((p == -1) && (errno != ECHILD)) {
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: master: waitpid error: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
		}

		/* remove the SHM file */
		if(group->shmid != -1) {
			status= unlink(group->shm_run_file);
			if(status != 0) {
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: master: Unable to remove SHM file!!: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
			}
			if(debug_level > 0) {
				write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master: Marking SHM segment for deletion");
			}
			struct shmid_ds *buf;
			buf = malloc(sizeof(struct shmid_ds));
			if(buf == NULL) {
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: master: malloc error: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
				exit(-1);
			}

			status=shmctl(group->shmid, IPC_RMID, buf);
			if(status !=0) {
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: master: SHM delete failed: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
				exit(-1);
			}
			else {
				if(debug_level > 0) {
					write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master: marked SHM segment for deletion");
				}
			}
			write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master:1");
			status=shmdt(group);
			if(status !=0) {
				write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master:1.5");
				snprintf(log_string, OCTOPUS_LOG_LEN, "ERROR: signal_handler: master: SHM detach failed: %s", strerror(errno));
				write_log(OCTOPUS_LOG_STD, log_string);
				exit(-1);
			}
			write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master:2");
			fclose(group->log_file);
		}
		else {
				write_log(OCTOPUS_LOG_STD, "NOTICE: signal_handler: master: termination signal received, no SHM segment detected so not detaching");
		}
	}
	exit(0);
}

