/*
 * Octopus Load Balancer - Standard stack implementation.
 * when a session has a state change (ready for read or write) it is handled then pushed onto this stack to set next state
 * after all sessions are serviced, their next state is set by popping and then setting state appropriately
 *
 * Copyright 2008 Alistair Reay <alreay1@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 */


int initialize_session_stack() {
	errno=0;
	SESSION **newcontents;
	newcontents = (SESSION **)(malloc(sizeof(SESSION *) * group->session_limit));
	if(newcontents == NULL) {
		snprintf(log_string, OCTOPUS_LOG_LEN,"ERROR: initialize_stack: unable to allocate memory for stack: %s", strerror(errno));
		write_log(OCTOPUS_LOG_EXIT, log_string);
	}
	session_stack.contents = newcontents;
	session_stack.top= -1;
	session_stack.maxsize= group->session_limit;
	return 0;
}

SESSION* stack_pop() {
	if(session_stack.top >= 0) {
		return session_stack.contents[(session_stack.top--)];
	}
	else {
		if(debug_level > 1) {
			snprintf(log_string, OCTOPUS_LOG_LEN,"WARNING: stack_pop: attempt to pop empty stack!");
			write_log(OCTOPUS_LOG_STD, log_string);
		}
		return NULL;
	}
	return NULL;
}
int stack_push (SESSION *s) {
	int i;
	for (i=0; i<= (session_stack.top); i++) {
		if(session_stack.contents[i]==s) {
			if(debug_level > 3) {
				snprintf(log_string, OCTOPUS_LOG_LEN,"DEBUG: stack_push: duplicate for session with serverfd %d and clientfd %d and clonefd %d\n", s->memberfd, s->clientfd, s->clonefd);
				write_log(OCTOPUS_LOG_STD, log_string);
			}
			return 0;
		}
	}
	if((session_stack.top+1) >= session_stack.maxsize) {
		snprintf(log_string, OCTOPUS_LOG_LEN,"WARNING: stack_push: stack full!");
		write_log(OCTOPUS_LOG_STD, log_string);
		return 1;
	}
	session_stack.top++;
	session_stack.contents[(session_stack.top)] = s;
	return 0;
}
